module Einutricional
  # Clase para almacenar los valores antropométricos de una persona
  class Patient < Person
    include Comparable
    attr_accessor :weight, :height, :waist, :hip

    # Constructor que recibe los datos de esa persona.
    # @note Solo name y surname son [String], el resto son [Fixnum]
    def initialize(name, surname, sex, age, weight, height, waist, hip, activity)
      super(name, surname, sex, age)
      @weight = weight
      @height = height
      @waist = waist
      @hip = hip
      @activity = activity
    end

    # Operador de comparación
    # @return -1, 0, 1 según si el imc del paciente 'self' es menor, igual o mayor que el del paciente con el que se compara
    def <=>(other)
      imc <=> other.imc
    end

    # Indice de Masa Corporal
    # @return [Fixnum] el IMC
    def imc
      (@weight.to_f / @height ** 2).round(2)
    end

    # Radio Cintura Cadera
    # @return [Fixnum] el RCC
    def rcc
      (@waist.to_f / @hip).round(2)
    end

    # Imprime los datos del paciente
    # @return [String] los datos del paciente
    def to_s
      "#{@surname}, #{@name}, #{@sex}, #{@age}, #{@weight}, #{@height}, #{@waist}, #{@hip}"
    end

    def peso_teorico_ideal
      (@height * 100 - 150) * 0.75 + 50
    end

    def gasto_energetico_basal
      geb = (10 * @weight) + (6.25 * @height * 100) - (5 * @age)
      geb += (@age == 'h' ? 5 : -161)
      geb.round(0)
    end

    def efecto_termogeno
      (gasto_energetico_basal * 0.10).round(0)
    end

    def gasto_actividad_fisica
      f = case @activity
          when 'reposo'
            0.0
          when 'ligera'
            0.12
          when 'moderada'
            0.27
          when 'intensa'
            0.54
          else
            0.0
          end
      (gasto_energetico_basal * f).round(0)
    end

    def gasto_energetico
      (gasto_energetico_basal + efecto_termogeno + gasto_actividad_fisica).round(0)
    end
  end
end
